/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "duskraylib.h"
#include "input.h"

typedef struct {
  int32_t key;
  uint8_t bind;
} inputkbmap_t;

typedef struct {
  GamepadButton button;
  uint8_t bind;
} inputgpmap_t;

inputkbmap_t INPUT_KB_MAP[] = {
  { KEY_UP, INPUT_BIND_UP },
  { KEY_W, INPUT_BIND_UP },
  { KEY_DOWN, INPUT_BIND_DOWN },
  { KEY_S, INPUT_BIND_DOWN },
  { KEY_LEFT, INPUT_BIND_LEFT },
  { KEY_A, INPUT_BIND_LEFT },
  { KEY_RIGHT, INPUT_BIND_RIGHT },
  { KEY_D, INPUT_BIND_RIGHT },

  { KEY_SPACE, INPUT_BIND_ACTION },
  { KEY_E, INPUT_BIND_ACTION },
  { KEY_ENTER, INPUT_BIND_ACTION },

  { KEY_ESCAPE, INPUT_BIND_CANCEL },
  { KEY_Q, INPUT_BIND_CANCEL },
  { KEY_BACKSPACE, INPUT_BIND_CANCEL },

  { 0, 0 }
};

inputgpmap_t INPUT_GP_MAP[] = {
  { GAMEPAD_BUTTON_LEFT_FACE_UP, INPUT_BIND_UP },
  { GAMEPAD_BUTTON_LEFT_FACE_DOWN, INPUT_BIND_DOWN },
  { GAMEPAD_BUTTON_LEFT_FACE_LEFT, INPUT_BIND_LEFT },
  { GAMEPAD_BUTTON_LEFT_FACE_RIGHT, INPUT_BIND_RIGHT },

  { GAMEPAD_BUTTON_RIGHT_FACE_RIGHT, INPUT_BIND_ACTION },
  { GAMEPAD_BUTTON_RIGHT_FACE_DOWN, INPUT_BIND_CANCEL },

  { 0, 0 }
};

uint8_t inputStateGet() {
  uint8_t state = 0;

  inputkbmap_t *kbMap = INPUT_KB_MAP;
  do {
    if(IsKeyDown(kbMap->key)) state |= kbMap->bind;
    kbMap++;
  } while(kbMap->key != 0);

  for(uint32_t i = 0; i < 32; i++) {
    if(!IsGamepadAvailable(i)) continue;
    
    inputgpmap_t *gpMap = INPUT_GP_MAP;
    do {
      if(IsGamepadButtonDown(i, gpMap->button)) state |= gpMap->bind;
      gpMap++;
    } while(gpMap->button != 0);
  }

  return state;
}